@@ -28,7 +28,15 @@
 
 extern hTab *liveRanges;
 extern hTab *iCodehTab;
+extern hTab *iCodeSeqhTab;
 
+int  notUsedInBlock (symbol *, eBBlock *, iCode *);
+bool allDefsOutOfRange (bitVect *, int, int);
 void computeLiveRanges (eBBlock **, int);
+void recomputeLiveRanges (eBBlock **, int);
+
+void setFromRange (operand *, int);
+void setToRange (operand *, int, bool);
+void hashiCodeKeys (eBBlock **, int);
 
 #endif
