@@ -30,12 +30,12 @@ struct pCodeOp;
 
 enum
 {
-	AOP_LIT = 1,
-	AOP_REG, AOP_DIR,
-	AOP_DPTR, AOP_DPTR2, AOP_R0, AOP_R1,
-	AOP_STK, AOP_IMMD, AOP_STR,
-	AOP_CRY, AOP_ACC,
-	AOP_PCODE
+  AOP_LIT = 1,
+  AOP_REG, AOP_DIR,
+  AOP_DPTR, AOP_DPTR2, AOP_R0, AOP_R1,
+  AOP_STK, AOP_IMMD, AOP_STR,
+  AOP_CRY, AOP_ACC,
+  AOP_PCODE
 
 };
 
@@ -45,47 +45,43 @@ enum
 typedef struct asmop
 {
 
-	short type;  /* can have values
-	                AOP_LIT    -  operand is a literal value
-	                AOP_REG    -  is in registers
-	                AOP_DIR    -  direct just a name
-	                AOP_DPTR   -  dptr contains address of operand
-	                AOP_DPTR2  -  dptr2 contains address of operand (DS80C390 only).
-	                AOP_R0/R1  -  r0/r1 contains address of operand               
-	                AOP_STK    -  should be pushed on stack this
-	                can happen only for the result
-	                AOP_IMMD   -  immediate value for eg. remateriazable 
-	                AOP_CRY    -  carry contains the value of this
-	                AOP_STR    -  array of strings
-	                AOP_ACC    -  result is in the acc:b pair
-	              */
-	short coff;			/* current offset */
-	short size;			/* total size */
-	unsigned code:1;		/* is in Code space */
-	unsigned paged:1;		/* in paged memory  */
-	unsigned freed:1;		/* already freed    */
-	union
-		{
-		value *aop_lit;		/* if literal */
-		regs *aop_reg[4];	/* array of registers */
-		char *aop_dir;		/* if direct  */
-		regs *aop_ptr;		/* either -> to r0 or r1 */
-		char *aop_immd;		/* if immediate others are implied */
-		int aop_stk;		/* stack offset when AOP_STK */
-		char *aop_str[4];	/* just a string array containing the location */
-/*		regs *aop_alloc_reg;     * points to a dynamically allocated register */
-		pCodeOp *pcop;
-	}
-	aopu;
+  short type;  /* can have values
+		  AOP_LIT    -  operand is a literal value
+		  AOP_REG    -  is in registers
+		  AOP_DIR    -  direct just a name
+		  AOP_DPTR   -  dptr contains address of operand
+		  AOP_DPTR2  -  dptr2 contains address of operand (DS80C390 only).
+		  AOP_R0/R1  -  r0/r1 contains address of operand               
+		  AOP_STK    -  should be pushed on stack this
+		  can happen only for the result
+		  AOP_IMMD   -  immediate value for eg. remateriazable 
+		  AOP_CRY    -  carry contains the value of this
+		  AOP_STR    -  array of strings
+		  AOP_ACC    -  result is in the acc:b pair
+	       */
+  short coff;	        /* current offset */
+  short size;	        /* total size */
+  unsigned code:1;      /* is in Code space */
+  unsigned paged:1;	/* in paged memory  */
+  unsigned freed:1;	/* already freed    */
+  union
+  {
+    value *aop_lit;     /* if literal */
+    regs *aop_reg[4];   /* array of registers */
+    char *aop_dir;      /* if direct  */
+    regs *aop_ptr;      /* either -> to r0 or r1 */
+    char *aop_immd;     /* if immediate others are implied */
+    int aop_stk;        /* stack offset when AOP_STK */
+    char *aop_str[4];   /* just a string array containing the location */
+    pCodeOp *pcop;
+  }
+  aopu;
 }
 asmop;
 
 void genpic14Code (iCode *);
 
-//extern char *fReturnpic14[];
-//extern char *fReturn390[];
 extern unsigned fReturnSizePic;
-//extern char **fReturn;
 
 
 #define AOP(op) op->aop
@@ -143,6 +139,7 @@ void emitpcode(PIC_OPCODE poc, pCodeOp *pcop);
 void emitpLabel(int key);
 void pic14_emitcode (char *inst,char *fmt, ...);
 void DEBUGpic14_emitcode (char *inst,char *fmt, ...);
+void pic14_emitDebuggerSymbol (char *);
 asmop *newAsmop (short type);
 bool pic14_sameRegs (asmop *aop1, asmop *aop2 );
 char *aopGet (asmop *aop, int offset, bool bit16, bool dname);
